
#include <QtWidgets>

class UpdateWidget : public QWidget {

public:    
    // A paintEvent implementation which only ever fills with lightblue
    void paintEvent(QPaintEvent *ev) override   {
        qDebug() << "paintEvent" << ev->rect();
        QPainter p(this);
        p.fillRect(ev->rect(), QColorConstants::Svg::lightblue);
    }
    
    // Call QWiget::update() with rect modified by an offset
    int m_minOffset = 0;
    int m_maxOffset = 30;
    int m_offset = 0;
    void updateWithOffset() {
        qDebug() << "updateWithOffset" << m_offset;
        update(QRect(40 - m_offset, 40 - m_offset, 50 + m_offset * 2, 50 + m_offset * 2));
    }

    // Change offset on timer
    void startAnim() {
        QTimer *timer = new QTimer(this);
        connect(timer, &QTimer::timeout, this, [this, timer]() {
            m_offset+=2;
            if (m_offset >= m_maxOffset)
                m_offset = m_minOffset;
            updateWithOffset();
        });
        timer->start(200);
    }
};

int main(int argc, char **argv) {
    
    QApplication app(argc, argv);
    
    // Expected behavior is that UpdateWidget is filled with solid lightblue,
    // actual behavior is that we sometimes get a black frame around the update
    // rect for fractional scale factors (try QT_SCALE_FACTOR=1.1 etc)

    UpdateWidget updateWidget;
    updateWidget.startAnim();
    updateWidget.resize(160, 160);
    updateWidget.show();

    app.exec();
}